/********************************************************************************************
(C) Jakub Borzdynski 2008
jotbeage@interia.pl
-----------------------------

********************************************************************************************/
#ifndef jb_btm222_cpp
#define jb_btm222_cpp
#include "btm222.h"

//konstruktor
btm222::btm222(){
	//konfiguracja modulu Bluetooth
	sbi(DDRD, 4) ; //sygnal RESET
	cbi(PORTD,4) ;
	_delay_ms(5) ;
	sbi(PORTD,4) ;
	_delay_ms(200) ;
	
	//CTS
	sbi(DDRD, 3) ;
	cbi(PORTD,3) ;
	
	//RTS
	cbi(DDRD, 2) ;
	cbi(PORTD,2) ;

	//konfiguracja portu Bluetooth
	sbi(UCSRB, TXEN) ; //wlacz nadajnik
	sbi(UCSRB, RXEN) ; //wlacz odbiornik
	UBRRL = 11 ; //predkosc 19200
	
	//ustaw porty
	sbi(PORTD,0) ;
	cbi(DDRD, 0) ;
	sbi(DDRD, 1) ;
	sbi(PORTD,1) ;
	
	//konfiguracja modulu
	if(dev.key() != 0){
		setupBTM() ;
	}
	
	//wlacz przerwanie
	sbi(SREG, SREG_I) ;
	sbi(UCSRB, RXCIE);
}

//ustawia nazwe oraz wylacza echo
void btm222::setupBTM(){
	//informacja
	for(char i=0 ; i<20 ; i++){
		_delay_ms(50) ;
		dev.statusLed(1)  ;
		_delay_ms(50) ;
		dev.statusLed(0)  ;
	}

	//nowe polecenie
	writeChar(13) ;
	//writeChar(10) ;
	_delay_ms(500) ;
	
	//wylacz echo modulu BTM
	char btmEcho[] = "ate0" ;
	for(unsigned char t=0 ; t<4 ; t++){
		writeChar(btmEcho[t]) ;
		_delay_ms(50) ;
	}
	writeChar(13) ;
	//writeChar(10) ;
	_delay_ms(500) ;
	
	//zapisz nazwe modulu
	char btmName[] = "atn=BlueSupply" ;
	for(unsigned char t=0 ; t<14 ; t++){
		writeChar(btmName[t]) ;
		_delay_ms(50) ;
	}
	writeChar(13) ;
	//writeChar(10) ;
	_delay_ms(500) ;
	
	//zapisz nazwe modulu
	char btmPin[] = "atp=1234" ;
	for(unsigned char t=0 ; t<8 ; t++){
		writeChar(btmPin[t]) ;
		_delay_ms(50) ;
	}
	writeChar(13) ;
	//writeChar(10) ;
	_delay_ms(500) ;
	
	//domyslne nazwy przekaznikow
	for(unsigned char slot=1 ; slot<5 ; slot++){
		unsigned char index = slot*22 ; //adres pamieci
		char name[] = {"DEVICE "} ;
		for(unsigned char p=0 ; p<7 ; p++){
			memory.write(index+p, name[p]) ;
		}
		memory.write(index+7, slot+'0') ;
		memory.write(index+8, 0) ;
	}
}

//zapisuje bajt danych do portu
void btm222::writeChar(char byte){
	//odczekaj na zwolnnienie bufora
	while(bit_is_clear(UCSRA, UDRE)){;} //czekaj na zwolnienie bufora
	
	//zapisz znak
	UDR = byte ;
}

//odczytuje ostatni bajt z portu
char btm222::readChar(){
	//zmienna pomocnicza
	char temp = 0xFF ; //"pusty" znak
	
	//pobranie znaku zbufora
	if(bufferIndex>0){
		//pobierz znak
		temp = buffer[0] ;
		
		//przesuwanie bufora
		cbi(SREG, SREG_I) ;
		for(unsigned char i=0 ; i<BTM222_BUFFER_SIZE-1 ; i++){
			buffer[i] = buffer[i+1] ;
		}
		bufferIndex--;
		sbi(SREG, SREG_I) ;
	}
	
	//zwroc rezultat
	return temp ;
}

//zwraca jeden, gdy jest jakis bajt do odczytania
bool btm222::isNewData(){
	return bufferIndex>0 ;
}

//przerwaniereturn bit_is_set(UCSRA, RXC) ;
void btm222::interrupt(){
	if(bufferIndex<BTM222_BUFFER_SIZE){
		buffer[bufferIndex] = UDR ; //odczytaj znak
		bufferIndex++; //zaznacz odebranie znaku
	}else{
		//brak miejsca w buforze
		char temp = UDR ; //pusty odczyt
		temp++ ; //zeby kompilator nie wyswietlal ostrzezen
	}
}

//kasuje zawartosc bufora
void btm222::clearBuffer(){
	cbi(SREG,  SREG_I) ;
	bufferIndex=0;
	sbi(SREG,  SREG_I) ;
}
	
btm222 btm ;

#endif